/**
 * <copyright>
 * 
 * Copyright (c) 2013 itemis and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     itemis - Initial API and implementation
 * 
 * </copyright>
 */
package org.safe.ide.natures;

import org.safe.ide.internal.Activator;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.sphinx.platform.util.ExtendedPlatform;
import org.eclipse.sphinx.platform.util.PlatformLogUtil;

/**
 * {@link IProjectNature project nature} for {@link IProject}s containing
 * SAFE models.
 */
public class SAFENature implements IProjectNature {

    /**
     * The id of this {@link IProjectNature project nature}.
     */
    public static final String ID = Activator.getPlugin().getSymbolicName()
	    + ".safenature"; //$NON-NLS-1$

    /**
     * The project to which this {@link IProjectNature project nature} applies.
     */
    private IProject project;

    /**
     * Default constructor.
     */
    public SAFENature() {
	// Do nothing
    }

    /**
     * Creates an instance of this {@link IProjectNature project nature} and
     * associates it with the given {@link IProject project}.
     * 
     * @param project
     *            The {@link IProject project} to be handled.
     */
    public SAFENature(IProject project) {
	setProject(project);
    }

    /**
     * Adds a {@link SAFENature SAFE nature} to the given {@link IProject
     * project}.
     * 
     * @param project
     *            The {@link IProject project} to be handled.
     * @param monitor
     *            A {@link IProgressMonitor progress monitor}, or
     *            <code>null</code> if progress reporting is not desired.
     * @throws CoreException
     *             If the {@link IProject project} does not exist or is not
     *             open.
     */
    public static void addTo(IProject project, IProgressMonitor monitor)
	    throws CoreException {
	ExtendedPlatform.addNature(project, ID, monitor);
    }

    /**
     * Removes the {@link SAFENature SAFE nature} from the given
     * {@link IProject project}.
     * 
     * @param project
     *            The {@link IProject project} to be handled.
     * @param monitor
     *            A {@link IProgressMonitor progress monitor}, or
     *            <code>null</code> if progress reporting is not desired.
     * @throws CoreException
     *             If the {@link IProject project} does not exist or is not
     *             open.
     */
    public static void removeFrom(IProject project, IProgressMonitor monitor)
	    throws CoreException {
	ExtendedPlatform.removeNature(project, ID, monitor);
    }

    /*
     * @see org.eclipse.core.resources.IProjectNature#configure()
     */
    public void configure() throws CoreException {
	// Do nothing
    }

    /*
     * @see org.eclipse.core.resources.IProjectNature#deconfigure()
     */
    public void deconfigure() throws CoreException {
	// Do nothing
    }

    /*
     * @see org.eclipse.core.resources.IProjectNature#getProject()
     */
    public IProject getProject() {
	return project;
    }

    /*
     * @see
     * org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core
     * .resources.IProject)
     */
    public void setProject(IProject project) {
	this.project = project;
    }
    
	/**
	 * * Checks if the specified {@linkplain IProject project} has the {@linkplain SAFENature#SAFE_NATURE_ID
	 * SAFE * nature}; that given project is not supposed to be <code>null</code> and is supposed to be *
	 * {@linkplain org.eclipse.core.resources.IResource#isAccessible() accessible}. Logs an error is case of *
	 * {@linkplain CoreException exception}. * * @param project *      A project that may have the SAFE
	 * nature; must not be <code>null</code> and must be *      <em>accessible</em>. * @return
	 * <ul>
	 * *    
	 * <li><tt><b>true</b>&nbsp;&nbsp;</tt> if the specified project has the SAFE nature;</li> *    
	 * <li><tt><b>false</b>&nbsp;</tt> otherwise.</li> *    
	 * </ul>
	 * 
	 */
	public static boolean has(IProject project) {
		Assert.isNotNull(project);
		Assert.isTrue(project.isAccessible());
		boolean hasSAFENature = false;
		try {
			hasSAFENature = project.hasNature(ID);
		} catch (CoreException ex) {
			PlatformLogUtil.logAsError(Activator.getDefault(), ex);
		}
		return hasSAFENature;
	}
}

