/**
 * <copyright>
 * 
 * Copyright (c) 2013 itemis and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     itemis - Initial API and implementation
 * 
 * </copyright>
 */
package org.safe.metamodel10.common.resource.impl;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.artop.eel.common.resource.ExtendedEObjectInputStream;
import org.artop.eel.common.resource.ExtendedEObjectOutputStream;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.common.CommonPlugin;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.resource.impl.BinaryResourceImpl;
import org.eclipse.emf.ecore.xmi.XMLLoad;
import org.eclipse.emf.ecore.xmi.XMLSave;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.osgi.util.NLS;
import org.eclipse.sphinx.emf.resource.ExtendedResource;
import org.safe.ide.preferences.ISAFEPreferences;
import org.safe.ide.util.SAFEObjectUtil;
import safe10.Activator;
import org.safe.metamodel10.common.internal.messages.Messages;
import org.safe.metamodel10.common.resource.SAFEURIFactory;

public class SAFEXMLResourceImpl extends XMLResourceImpl {
    
    /**
     * An option which set on the resource during parsing and used in the
     * SAFEResourceHandler. The value <code>Boolean.TRUE</code> is to force
     * iteration on the resource.
     */
    public static final String OPTION_APPLY_RESOURCE_HANDLER = "APPLY_RESOURCE_HANDLER"; //$NON-NLS-1$

    /**
	 * {@link Adapter} providing SAFE-specific implementation of {@link ExtendedResource extended resource} services.
	 * 
	 * @see ExtendedResource
	 */
	protected ExtendedSAFEResourceAdapter extendedResource;
	
    /**
     * Creates an SAFEXMLResourceImpl with the given URI.
     * 
     * @param uri
     *            The URI defining the location of the SAFEXMLResourceImpl.
     */
    public SAFEXMLResourceImpl(URI uri) {
	super(uri);
	
	// Install adapter providing SAFE-specific implementation of extended resource services
	extendedResource = new ExtendedSAFEResourceAdapter();
	eAdapters().add(extendedResource);
    }

    @Override
    public EObject getEObject(String uriFragment) {
	if (getContents().size() > 0) {
	     return SAFEURIFactory.getEObject(getContents().get(0), uriFragment);
	}
	return null;
    }

     @Override
     public String getURIFragment(EObject object) {
	 return SAFEURIFactory.getURIFragment(object);
     }

     
     @Override
 	protected void unloaded(InternalEObject internalEObject) {
 		// Delegate to implementation provided by extended resource for enabling memory-optimized unloading
 		extendedResource.unloaded(internalEObject);
 	}
     


	/**
	 * Loads the inputStream from a binary resource. Currently (EMF 2.6), the only supported option is:
	 * 
	 * @see BinaryResourceImpl#OPTION_BUFFER_CAPACITY
	 * @param binaryFile
	 *            The binary file to be loaded.
	 * @param options
	 *            The options for the binary resource.
	 * @throws IOException
	 *             Any IOException raised when the binary resource loads the input stream.
	 */
	protected void loadFromBinaryFile(File binaryFile, Map<?, ?> options) throws IOException {
		InputStream inputStream = new FileInputStream(binaryFile);
		int bufferCapacity = BinaryResourceImpl.getBufferCapacity(options);
		if (bufferCapacity > 0) {
			inputStream = new BufferedInputStream(inputStream, bufferCapacity);
		}

		ExtendedEObjectInputStream eObjectInputStream = new ExtendedEObjectInputStream(inputStream, options) {

			@Override
			protected void preLoadFeatureValues(InternalEObject internalEObject, Map<String, String> destValues) throws IOException {
				// Setting the dest values shall be done after the object is fully loaded because
				// when a reference is set it's previously set dest will be cleared.
				int size = readCompressedInt();
				for (int i = 0; i < size; i++) {
					String key = readString();
					String dest = readString();
					destValues.put(key, dest);
				}
			}

			@Override
			protected void postLoadFeatureValues(InternalEObject internalEObject, Map<String, String> destValues) throws IOException {
				if (destValues != null) {
					// Now it's safe to set the dest values
					for (String key : destValues.keySet()) {
						SAFEObjectUtil.setDest(internalEObject, key, destValues.get(key));
					}
				}
			}
		};
		eObjectInputStream.loadResource(this);
	}
	
	/**
	 * Gets the binary file associated with given xml file.
	 * 
	 * @param xmlFile
	 *            The xml file.
	 * @return As specified above.
	 */
	public static File getBinaryFile(File xmlFile) {
		return getBinaryPath(xmlFile).toFile();
	}

	/**
	 * Gets the path of the binary file associated with given xml file.
	 * 
	 * @param xmlFile
	 *            The xml file.
	 * @return As specified above.
	 */
	private static IPath getBinaryPath(File xmlFile) {
		// FIXME Change this and store in the project so that clean in builders destroy the cache
		return Platform.getStateLocation(Activator.getPlugin().getBundle()).append(new Path(xmlFile.getAbsolutePath()).addFileExtension("arbin")); //$NON-NLS-1$
	}

	/**
	 * Returns true if and only if the binary version of given xml file has a timestamp older than given xml file
	 * timestamp.
	 * 
	 * @param xmlFile
	 *            A non null file.
	 * @return As specified above.
	 */
	protected static boolean isBinaryResourceAsOldAs(File xmlFile) {
		long xmlStamp = xmlFile.lastModified();
		long binaryStamp = getBinaryPath(xmlFile).toFile().lastModified();
		return xmlStamp != IResource.NULL_STAMP && binaryStamp >= xmlStamp;
	}

	/**
	 * Gets the file that corresponds to the uri of this resource.
	 * 
	 * @return As specifed above.
	 */
	protected File getXMLFile() {
		File xmlFile = null;
		URI convertedURI = uri;
		if (!uri.isFile() && uri.scheme() != null && uri.scheme().matches("[A-Z]")) { //$NON-NLS-1$
			convertedURI = URI.createFileURI(uri.toString());
		}
		convertedURI = CommonPlugin.asLocalURI(uri);
		if (convertedURI.isFile()) {
			xmlFile = new File(convertedURI.toFileString());
		}
		return xmlFile;
	}

    /*
     * @see
     * org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl#doLoad(java.io.InputStream
     * , java.util.Map)
     */
    @Override
	public void doLoad(InputStream inputStream, Map<?, ?> options) throws IOException {
		// Bugfix: Bug 1406 - Problems with Serialization of extended models with binary resources
		// https://www.artop.org/bugs/show_bug.cgi?id=1406
		// Replicated code from parent to take into account binary serialization between preLoad() and
		// postLoad() actions which process extensions.

	boolean hasLoadedFromBinaryResource = false;

	XMLLoad xmlLoad = createXMLLoad();

	if (options == null) {
	    options = Collections.EMPTY_MAP;
	}

		ResourceHandler handler = (ResourceHandler) options.get(OPTION_RESOURCE_HANDLER);
		if (handler != null) {
			handler.preLoad(this, inputStream, options);
		}

		if (useBinaryResources()) {
			File xmlFile = getXMLFile();
			if (isBinaryResourceAsOldAs(xmlFile)) {
				loadFromBinaryFile(getBinaryFile(xmlFile), options);
				hasLoadedFromBinaryResource = true;
			}
		}

		if (!hasLoadedFromBinaryResource) {
			xmlLoad.load(this, inputStream, options);
			xmlLoad = null;

			// Paradoxically, when loading from XML, save to binary.
			// This supports the use case of an imported project with many existing files will not be changed.
			if (useBinaryResources()) {
				saveToBinaryResource(options);
			}
		}

		if (handler != null) {
			handler.postLoad(this, inputStream, options);
		}

		EditingDomain editingDomain = TransactionUtil.getEditingDomain(this);
	}

    /*
     * @see
     * org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl#doSave(java.io.OutputStream
     * , java.util.Map)
     */
    @Override
	public void doSave(OutputStream outputStream, Map<?, ?> options) throws IOException {
		// Bugfix: Bug 1406 - Problems with Serialization of extended models with binary resources
		// https://www.artop.org/bugs/show_bug.cgi?id=1406
		// Replicated code from parent to take into account binary serialization between preSave() and
		// postSave() actions which process extensions.

	XMLSave xmlSave = createXMLSave();

	if (options == null) {
	    options = Collections.EMPTY_MAP;
	}

		ResourceHandler handler = (ResourceHandler) options.get(OPTION_RESOURCE_HANDLER);

	if (handler != null) {
	    handler.preSave(this, outputStream, options);
	}

	xmlSave.save(this, outputStream, options);

		// Saving as binary resource is now enclosed between preSave and postSave action
		if (useBinaryResources()) {
			saveToBinaryResource(options);
		}

		if (handler != null) {
			handler.postSave(this, outputStream, options);
		}

		EditingDomain editingDomain = TransactionUtil.getEditingDomain(this);
	}

	/**
	 * Saves current resource content to a binary file.
	 * 
	 * @param options
	 *            A map that may define the buffer capacity.
	 * @throws IOException
	 *             Any IOException raised when saving.
	 * @since 3.0
	 */
	protected void saveToBinaryResource(Map<?, ?> options) throws IOException {
		File xmlFile = getXMLFile();
		File binaryFile = getBinaryFile(xmlFile);
		File parentFile = binaryFile.getParentFile();
		if (!parentFile.exists()) {
			parentFile.mkdirs();
		}
		OutputStream outputStream = new FileOutputStream(binaryFile);

		// The reason for this unfortunate situation is that content from this AutosarXMLResourceImpl is passed directly
		// to the object output stream without the need of temporary memory duplication.
		boolean buffer = true;
		int bufferCapacity = BinaryResourceImpl.getBufferCapacity(options);
		if (bufferCapacity > 0) {
			outputStream = new BufferedOutputStream(outputStream, bufferCapacity);
		} else {
			buffer = false;
		}

		try {
			ExtendedEObjectOutputStream eObjectOutputStream = new ExtendedEObjectOutputStream(outputStream, options) {

				@Override
				protected void handleAdvancedFeatures(InternalEObject internalEObject, Check check) throws IOException {
					Map<String, String> eObjectAbstractDests = SAFEObjectUtil.getEObjectAbstractDests(internalEObject);
					if (eObjectAbstractDests != null) {
						Set<Entry<String, String>> entrySet = eObjectAbstractDests.entrySet();
						int size = entrySet.size();
						writeCompressedInt(size);
						for (Entry<String, String> entry : entrySet) {
							writeString(entry.getKey());
							writeString(entry.getValue());
						}
					} else {
						writeCompressedInt(0);
					}
				}
			};
			eObjectOutputStream.saveResource(this);
		} finally {
			if (buffer) {
				outputStream.flush();
			}
			outputStream.close();
		}
	}

	@Override
	protected void doUnload() {
		super.doUnload();
	}

	/**
	 * Recursively deletes the content of given directory.
	 * 
	 * @param dir
	 *            The directory the content of which must be deleted.
	 * @param monitor
	 *            The progress monitor.
	 */
	private static void deleteDirectoryContent(File dir, IProgressMonitor monitor) {
		try {
			final int mult = 1000;
			String taskName = NLS.bind(Messages.clearing_binaryResources, dir.toString());
			if (dir.isDirectory()) {
				File[] listFiles = dir.listFiles();
				monitor.beginTask(taskName, listFiles.length * mult);
				for (File f : listFiles) {
					if (!f.isDirectory()) {
						f.delete();
						monitor.worked(mult);
					}
				}
				for (File f : listFiles) {
					if (f.isDirectory()) {
						deleteDirectoryContent(f, new SubProgressMonitor(monitor, mult - 1));
						f.delete();
						monitor.worked(1);
					}
				}
			} else {
				monitor.beginTask(taskName, 1);
				monitor.worked(1);
			}
		} finally {
			monitor.done();
		}

	}

	public static void clearCache(IProgressMonitor monitor) {
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}

		IPath stateLocation = Platform.getStateLocation(Activator.getPlugin().getBundle());
		deleteDirectoryContent(stateLocation.toFile(), monitor);
	}

	protected boolean useBinaryResources() {
		// preferences cannot be accessed in standalone mode
		if (!Platform.isRunning()) {
			return false;
		}
		return ISAFEPreferences.PREF_USE_BINARY_RESOURCE.get();
	}
}

